@@ -2,8 +2,6 @@ from __future__ import absolute_import, division, print_function
 
 import sys
 
-import pytest
-
 from hypothesis import HealthCheck, settings
 
 
@@ -15,23 +13,19 @@ def pytest_configure(config):
     settings.load_profile("patience")
 
 
-@pytest.fixture(scope="session")
-def C():
-    """
-    Return a simple but fully featured attrs class with an x and a y attribute.
-    """
-    import attr
-
-    @attr.s
-    class C(object):
-        x = attr.ib()
-        y = attr.ib()
-
-    return C
-
-
 collect_ignore = []
 if sys.version_info[:2] < (3, 6):
     collect_ignore.extend(
-        ["tests/test_annotations.py", "tests/test_init_subclass.py"]
+        [
+            "tests/test_annotations.py",
+            "tests/test_hooks.py",
+            "tests/test_init_subclass.py",
+            "tests/test_next_gen.py",
+        ]
+    )
+if sys.version_info[:2] >= (3, 10):
+    collect_ignore.extend(
+        [
+            "tests/test_mypy.yml",
+        ]
     )
