@@ -5,6 +5,7 @@ Tests for dunder methods from `attrib._make`.
 from __future__ import absolute_import, division, print_function
 
 import copy
+import pickle
 
 import pytest
 
@@ -16,8 +17,9 @@ import attr
 from attr._make import (
     NOTHING,
     Factory,
-    _add_init,
     _add_repr,
+    _is_slot_cls,
+    _make_init,
     _Nothing,
     fields,
     make_class,
@@ -27,19 +29,46 @@ from attr.validators import instance_of
 from .utils import simple_attr, simple_class
 
 
-CmpC = simple_class(cmp=True)
-CmpCSlots = simple_class(cmp=True, slots=True)
+EqC = simple_class(eq=True)
+EqCSlots = simple_class(eq=True, slots=True)
+OrderC = simple_class(order=True)
+OrderCSlots = simple_class(order=True, slots=True)
 ReprC = simple_class(repr=True)
 ReprCSlots = simple_class(repr=True, slots=True)
 
+
+@attr.s(eq=True)
+class EqCallableC(object):
+    a = attr.ib(eq=str.lower, order=False)
+    b = attr.ib(eq=True)
+
+
+@attr.s(eq=True, slots=True)
+class EqCallableCSlots(object):
+    a = attr.ib(eq=str.lower, order=False)
+    b = attr.ib(eq=True)
+
+
+@attr.s(order=True)
+class OrderCallableC(object):
+    a = attr.ib(eq=True, order=str.lower)
+    b = attr.ib(order=True)
+
+
+@attr.s(order=True, slots=True)
+class OrderCallableCSlots(object):
+    a = attr.ib(eq=True, order=str.lower)
+    b = attr.ib(order=True)
+
+
 # HashC is hashable by explicit definition while HashCSlots is hashable
 # implicitly.  The "Cached" versions are the same, except with hash code
 # caching enabled
 HashC = simple_class(hash=True)
-HashCSlots = simple_class(hash=None, cmp=True, frozen=True, slots=True)
+HashCSlots = simple_class(hash=None, eq=True, frozen=True, slots=True)
 HashCCached = simple_class(hash=True, cache_hash=True)
 HashCSlotsCached = simple_class(
-    hash=None, cmp=True, frozen=True, slots=True, cache_hash=True
+    hash=None, eq=True, frozen=True, slots=True, cache_hash=True
 )
 # the cached hash code is stored slightly differently in this case
 # so it needs to be tested separately
@@ -48,6 +77,29 @@ HashCFrozenNotSlotsCached = simple_class(
 )
 
 
+def _add_init(cls, frozen):
+    """
+    Add a __init__ method to *cls*.  If *frozen* is True, make it immutable.
+
+    This function used to be part of _make.  It wasn't used anymore however
+    the tests for it are still useful to test the behavior of _make_init.
+    """
+    cls.__init__ = _make_init(
+        cls,
+        cls.__attrs_attrs__,
+        getattr(cls, "__attrs_pre_init__", False),
+        getattr(cls, "__attrs_post_init__", False),
+        frozen,
+        _is_slot_cls(cls),
+        cache_hash=False,
+        base_attr_map={},
+        is_exc=False,
+        has_global_on_setattr=False,
+        attrs_init=False,
+    )
+    return cls
+
+
 class InitC(object):
     __attrs_attrs__ = [simple_attr("a"), simple_attr("b")]
 
@@ -55,23 +107,23 @@ class InitC(object):
 InitC = _add_init(InitC, False)
 
 
-class TestAddCmp(object):
+class TestEqOrder(object):
     """
-    Tests for `_add_cmp`.
+    Tests for eq and order related methods.
     """
 
     @given(booleans())
-    def test_cmp(self, slots):
+    def test_eq_ignore_attrib(self, slots):
         """
-        If `cmp` is False, ignore that attribute.
+        If `eq` is False for an attribute, ignore that attribute.
         """
         C = make_class(
-            "C", {"a": attr.ib(cmp=False), "b": attr.ib()}, slots=slots
+            "C", {"a": attr.ib(eq=False), "b": attr.ib()}, slots=slots
         )
 
         assert C(1, 2) == C(2, 2)
 
-    @pytest.mark.parametrize("cls", [CmpC, CmpCSlots])
+    @pytest.mark.parametrize("cls", [EqC, EqCSlots])
     def test_equal(self, cls):
         """
         Equal objects are detected as equal.
@@ -79,7 +131,17 @@ class TestAddCmp(object):
         assert cls(1, 2) == cls(1, 2)
         assert not (cls(1, 2) != cls(1, 2))
 
-    @pytest.mark.parametrize("cls", [CmpC, CmpCSlots])
+    @pytest.mark.parametrize("cls", [EqCallableC, EqCallableCSlots])
+    def test_equal_callable(self, cls):
+        """
+        Equal objects are detected as equal.
+        """
+        assert cls("Test", 1) == cls("test", 1)
+        assert cls("Test", 1) != cls("test", 2)
+        assert not (cls("Test", 1) != cls("test", 1))
+        assert not (cls("Test", 1) == cls("test", 2))
+
+    @pytest.mark.parametrize("cls", [EqC, EqCSlots])
     def test_unequal_same_class(self, cls):
         """
         Unequal objects of correct type are detected as unequal.
@@ -87,21 +149,31 @@ class TestAddCmp(object):
         assert cls(1, 2) != cls(2, 1)
         assert not (cls(1, 2) == cls(2, 1))
 
-    @pytest.mark.parametrize("cls", [CmpC, CmpCSlots])
+    @pytest.mark.parametrize("cls", [EqCallableC, EqCallableCSlots])
+    def test_unequal_same_class_callable(self, cls):
+        """
+        Unequal objects of correct type are detected as unequal.
+        """
+        assert cls("Test", 1) != cls("foo", 2)
+        assert not (cls("Test", 1) == cls("foo", 2))
+
+    @pytest.mark.parametrize(
+        "cls", [EqC, EqCSlots, EqCallableC, EqCallableCSlots]
+    )
     def test_unequal_different_class(self, cls):
         """
         Unequal objects of different type are detected even if their attributes
         match.
         """
 
-        class NotCmpC(object):
+        class NotEqC(object):
             a = 1
             b = 2
 
-        assert cls(1, 2) != NotCmpC()
-        assert not (cls(1, 2) == NotCmpC())
+        assert cls(1, 2) != NotEqC()
+        assert not (cls(1, 2) == NotEqC())
 
-    @pytest.mark.parametrize("cls", [CmpC, CmpCSlots])
+    @pytest.mark.parametrize("cls", [OrderC, OrderCSlots])
     def test_lt(self, cls):
         """
         __lt__ compares objects as tuples of attribute values.
@@ -113,14 +185,28 @@ class TestAddCmp(object):
         ]:
             assert cls(*a) < cls(*b)
 
-    @pytest.mark.parametrize("cls", [CmpC, CmpCSlots])
+    @pytest.mark.parametrize("cls", [OrderCallableC, OrderCallableCSlots])
+    def test_lt_callable(self, cls):
+        """
+        __lt__ compares objects as tuples of attribute values.
+        """
+        # Note: "A" < "a"
+        for a, b in [
+            (("test1", 1), ("Test1", 2)),
+            (("test0", 1), ("Test1", 1)),
+        ]:
+            assert cls(*a) < cls(*b)
+
+    @pytest.mark.parametrize(
+        "cls", [OrderC, OrderCSlots, OrderCallableC, OrderCallableCSlots]
+    )
     def test_lt_unordable(self, cls):
         """
         __lt__ returns NotImplemented if classes differ.
         """
         assert NotImplemented == (cls(1, 2).__lt__(42))
 
-    @pytest.mark.parametrize("cls", [CmpC, CmpCSlots])
+    @pytest.mark.parametrize("cls", [OrderC, OrderCSlots])
     def test_le(self, cls):
         """
         __le__ compares objects as tuples of attribute values.
@@ -134,14 +220,30 @@ class TestAddCmp(object):
         ]:
             assert cls(*a) <= cls(*b)
 
-    @pytest.mark.parametrize("cls", [CmpC, CmpCSlots])
+    @pytest.mark.parametrize("cls", [OrderCallableC, OrderCallableCSlots])
+    def test_le_callable(self, cls):
+        """
+        __le__ compares objects as tuples of attribute values.
+        """
+        # Note: "A" < "a"
+        for a, b in [
+            (("test1", 1), ("Test1", 1)),
+            (("test1", 1), ("Test1", 2)),
+            (("test0", 1), ("Test1", 1)),
+            (("test0", 2), ("Test1", 1)),
+        ]:
+            assert cls(*a) <= cls(*b)
+
+    @pytest.mark.parametrize(
+        "cls", [OrderC, OrderCSlots, OrderCallableC, OrderCallableCSlots]
+    )
     def test_le_unordable(self, cls):
         """
         __le__ returns NotImplemented if classes differ.
         """
         assert NotImplemented == (cls(1, 2).__le__(42))
 
-    @pytest.mark.parametrize("cls", [CmpC, CmpCSlots])
+    @pytest.mark.parametrize("cls", [OrderC, OrderCSlots])
     def test_gt(self, cls):
         """
         __gt__ compares objects as tuples of attribute values.
@@ -153,14 +255,28 @@ class TestAddCmp(object):
         ]:
             assert cls(*a) > cls(*b)
 
-    @pytest.mark.parametrize("cls", [CmpC, CmpCSlots])
+    @pytest.mark.parametrize("cls", [OrderCallableC, OrderCallableCSlots])
+    def test_gt_callable(self, cls):
+        """
+        __gt__ compares objects as tuples of attribute values.
+        """
+        # Note: "A" < "a"
+        for a, b in [
+            (("Test1", 2), ("test1", 1)),
+            (("Test1", 1), ("test0", 1)),
+        ]:
+            assert cls(*a) > cls(*b)
+
+    @pytest.mark.parametrize(
+        "cls", [OrderC, OrderCSlots, OrderCallableC, OrderCallableCSlots]
+    )
     def test_gt_unordable(self, cls):
         """
         __gt__ returns NotImplemented if classes differ.
         """
         assert NotImplemented == (cls(1, 2).__gt__(42))
 
-    @pytest.mark.parametrize("cls", [CmpC, CmpCSlots])
+    @pytest.mark.parametrize("cls", [OrderC, OrderCSlots])
     def test_ge(self, cls):
         """
         __ge__ compares objects as tuples of attribute values.
@@ -174,7 +290,23 @@ class TestAddCmp(object):
         ]:
             assert cls(*a) >= cls(*b)
 
-    @pytest.mark.parametrize("cls", [CmpC, CmpCSlots])
+    @pytest.mark.parametrize("cls", [OrderCallableC, OrderCallableCSlots])
+    def test_ge_callable(self, cls):
+        """
+        __ge__ compares objects as tuples of attribute values.
+        """
+        # Note: "A" < "a"
+        for a, b in [
+            (("Test1", 1), ("test1", 1)),
+            (("Test1", 2), ("test1", 1)),
+            (("Test1", 1), ("test0", 1)),
+            (("Test1", 1), ("test0", 2)),
+        ]:
+            assert cls(*a) >= cls(*b)
+
+    @pytest.mark.parametrize(
+        "cls", [OrderC, OrderCSlots, OrderCallableC, OrderCallableCSlots]
+    )
     def test_ge_unordable(self, cls):
         """
         __ge__ returns NotImplemented if classes differ.
@@ -205,6 +337,21 @@ class TestAddRepr(object):
         """
         assert "C(a=1, b=2)" == repr(cls(1, 2))
 
+    def test_custom_repr_works(self):
+        """
+        repr returns a sensible value for attributes with a custom repr
+        callable.
+        """
+
+        def custom_repr(value):
+            return "foo:" + str(value)
+
+        @attr.s
+        class C(object):
+            a = attr.ib(repr=custom_repr)
+
+        assert "C(a=foo:1)" == repr(C(1))
+
     def test_infinite_recursion(self):
         """
         In the presence of a cyclic graph, repr will emit an ellipsis and not
@@ -271,6 +418,33 @@ class TestAddRepr(object):
         ) == e.value.args[0]
 
 
+# these are for use in TestAddHash.test_cache_hash_serialization
+# they need to be out here so they can be un-pickled
+@attr.attrs(hash=True, cache_hash=False)
+class HashCacheSerializationTestUncached(object):
+    foo_value = attr.ib()
+
+
+@attr.attrs(hash=True, cache_hash=True)
+class HashCacheSerializationTestCached(object):
+    foo_value = attr.ib()
+
+
+@attr.attrs(slots=True, hash=True, cache_hash=True)
+class HashCacheSerializationTestCachedSlots(object):
+    foo_value = attr.ib()
+
+
+class IncrementingHasher(object):
+    def __init__(self):
+        self.hash_value = 100
+
+    def __hash__(self):
+        rv = self.hash_value
+        self.hash_value += 1
+        return rv
+
+
 class TestAddHash(object):
     """
     Tests for `_add_hash`.
@@ -310,7 +484,7 @@ class TestAddHash(object):
         # unhashable case
         with pytest.raises(TypeError) as e:
             make_class(
-                "C", {}, hash=None, cmp=True, frozen=False, cache_hash=True
+                "C", {}, hash=None, eq=True, frozen=False, cache_hash=True
             )
         assert exc_args == e.value.args
 
@@ -343,13 +517,13 @@ class TestAddHash(object):
         assert hash(C(1, 2)) == hash(C(2, 2))
 
     @given(booleans())
-    def test_hash_attribute_mirrors_cmp(self, cmp):
+    def test_hash_attribute_mirrors_eq(self, eq):
         """
-        If `hash` is None, the hash generation mirrors `cmp`.
+        If `hash` is None, the hash generation mirrors `eq`.
         """
-        C = make_class("C", {"a": attr.ib(cmp=cmp)}, cmp=True, frozen=True)
+        C = make_class("C", {"a": attr.ib(eq=eq)}, eq=True, frozen=True)
 
-        if cmp:
+        if eq:
             assert C(1) != C(2)
             assert hash(C(1)) != hash(C(2))
             assert hash(C(1)) == hash(C(1))
@@ -358,18 +532,18 @@ class TestAddHash(object):
             assert hash(C(1)) == hash(C(2))
 
     @given(booleans())
-    def test_hash_mirrors_cmp(self, cmp):
+    def test_hash_mirrors_eq(self, eq):
         """
-        If `hash` is None, the hash generation mirrors `cmp`.
+        If `hash` is None, the hash generation mirrors `eq`.
         """
-        C = make_class("C", {"a": attr.ib()}, cmp=cmp, frozen=True)
+        C = make_class("C", {"a": attr.ib()}, eq=eq, frozen=True)
 
         i = C(1)
 
         assert i == i
         assert hash(i) == hash(i)
 
-        if cmp:
+        if eq:
             assert C(1) == C(1)
             assert hash(C(1)) == hash(C(1))
         else:
@@ -453,6 +627,88 @@ class TestAddHash(object):
         assert 2 == uncached_instance.hash_counter.times_hash_called
         assert 1 == cached_instance.hash_counter.times_hash_called
 
+    @pytest.mark.parametrize("cache_hash", [True, False])
+    @pytest.mark.parametrize("frozen", [True, False])
+    @pytest.mark.parametrize("slots", [True, False])
+    def test_copy_hash_cleared(self, cache_hash, frozen, slots):
+        """
+        Test that the default hash is recalculated after a copy operation.
+        """
+
+        kwargs = dict(frozen=frozen, slots=slots, cache_hash=cache_hash)
+
+        # Give it an explicit hash if we don't have an implicit one
+        if not frozen:
+            kwargs["hash"] = True
+
+        @attr.s(**kwargs)
+        class C(object):
+            x = attr.ib()
+
+        a = C(IncrementingHasher())
+        # Ensure that any hash cache would be calculated before copy
+        orig_hash = hash(a)
+        b = copy.deepcopy(a)
+
+        if kwargs["cache_hash"]:
+            # For cache_hash classes, this call is cached
+            assert orig_hash == hash(a)
+
+        assert orig_hash != hash(b)
+
+    @pytest.mark.parametrize(
+        "klass,cached",
+        [
+            (HashCacheSerializationTestUncached, False),
+            (HashCacheSerializationTestCached, True),
+            (HashCacheSerializationTestCachedSlots, True),
+        ],
+    )
+    def test_cache_hash_serialization_hash_cleared(self, klass, cached):
+        """
+        Tests that the hash cache is cleared on deserialization to fix
+        https://github.com/python-attrs/attrs/issues/482 .
+
+        This test is intended to guard against a stale hash code surviving
+        across serialization (which may cause problems when the hash value
+        is different in different interpreters).
+        """
+
+        obj = klass(IncrementingHasher())
+        original_hash = hash(obj)
+        obj_rt = self._roundtrip_pickle(obj)
+
+        if cached:
+            assert original_hash == hash(obj)
+
+        assert original_hash != hash(obj_rt)
+
+    @pytest.mark.parametrize("frozen", [True, False])
+    def test_copy_two_arg_reduce(self, frozen):
+        """
+        If __getstate__ returns None, the tuple returned by object.__reduce__
+        won't contain the state dictionary; this test ensures that the custom
+        __reduce__ generated when cache_hash=True works in that case.
+        """
+
+        @attr.s(frozen=frozen, cache_hash=True, hash=True)
+        class C(object):
+            x = attr.ib()
+
+            def __getstate__(self):
+                return None
+
+        # By the nature of this test it doesn't really create an object that's
+        # in a valid state - it basically does the equivalent of
+        # `object.__new__(C)`, so it doesn't make much sense to assert anything
+        # about the result of the copy. This test will just check that it
+        # doesn't raise an *error*.
+        copy.deepcopy(C(1))
+
+    def _roundtrip_pickle(self, obj):
+        pickle_str = pickle.dumps(obj)
+        return pickle.loads(pickle_str)
+
 
 class TestAddInit(object):
     """
@@ -473,9 +729,8 @@ class TestAddInit(object):
         with pytest.raises(TypeError) as e:
             C(a=1, b=2)
 
-        assert (
+        assert e.value.args[0].endswith(
             "__init__() got an unexpected keyword argument 'a'"
-            == e.value.args[0]
         )
 
     @given(booleans(), booleans())
@@ -658,3 +913,55 @@ class TestNothing(object):
         assert _Nothing() == _Nothing() == NOTHING
         assert not (_Nothing() != _Nothing())
         assert 1 != _Nothing()
+
+    def test_false(self):
+        """
+        NOTHING evaluates as falsey.
+        """
+        assert not NOTHING
+        assert False is bool(NOTHING)
+
+
+@attr.s(hash=True, order=True)
+class C(object):
+    pass
+
+
+# Store this class so that we recreate it.
+OriginalC = C
+
+
+@attr.s(hash=True, order=True)
+class C(object):
+    pass
+
+
+class TestFilenames(object):
+    def test_filenames(self):
+        """
+        The created dunder methods have a "consistent" filename.
+        """
+        assert (
+            OriginalC.__init__.__code__.co_filename
+            == "<attrs generated init tests.test_dunders.C>"
+        )
+        assert (
+            OriginalC.__eq__.__code__.co_filename
+            == "<attrs generated eq tests.test_dunders.C>"
+        )
+        assert (
+            OriginalC.__hash__.__code__.co_filename
+            == "<attrs generated hash tests.test_dunders.C>"
+        )
+        assert (
+            C.__init__.__code__.co_filename
+            == "<attrs generated init tests.test_dunders.C-2>"
+        )
+        assert (
+            C.__eq__.__code__.co_filename
+            == "<attrs generated eq tests.test_dunders.C-2>"
+        )
+        assert (
+            C.__hash__.__code__.co_filename
+            == "<attrs generated hash tests.test_dunders.C-2>"
+        )
